@@ -29,10 +29,13 @@
 #  - hipchat notifications by @ktsaou #1561
 #  - fleep notifications by @Ferroin
 #  - prowlapp.com notifications by @Ferroin
+#  - irc notifications by @manosf
 #  - custom notifications by @ktsaou
 #  - syslog messages by @Ferroin
 #  - Microsoft Team notification by @tioumen
 #  - RocketChat notifications by @Hermsi1337 #3777
+#  - Google Hangouts Chat notifications by @EnzoAkira and @hendrikhofstadt
+#  - Dynatrace Event by @illumine
 
 # -----------------------------------------------------------------------------
 # testing notifications
@@ -163,9 +166,13 @@ custom
 msteam
 kavenegar
 prowl
+irc
 awssns
 rocketchat
 sms
+hangouts
+dynatrace
+matrix
 "
 
 # -----------------------------------------------------------------------------
@@ -189,7 +196,7 @@ fi
 [ -z "${NETDATA_STOCK_CONFIG_DIR}" ] && NETDATA_STOCK_CONFIG_DIR="/usr/lib/netdata/conf.d"
 [ -z "${NETDATA_CACHE_DIR}" ] && NETDATA_CACHE_DIR="/var/cache/netdata"
 [ -z "${NETDATA_REGISTRY_URL}" ] && NETDATA_REGISTRY_URL="https://registry.my-netdata.io"
-[ -z "${NETDATA_REGISTRY_CLOUD_BASE_URL}" ] && NETDATA_REGISTRY_CLOUD_BASE_URL="https://netdata.cloud"
+[ -z "${NETDATA_REGISTRY_CLOUD_BASE_URL}" ] && NETDATA_REGISTRY_CLOUD_BASE_URL="https://app.netdata.cloud"
 
 # -----------------------------------------------------------------------------
 # parse command line parameters
@@ -345,6 +352,10 @@ FLEEP_SENDER="${host}"
 # Amazon SNS configs
 AWSSNS_MESSAGE_FORMAT=
 
+# Matrix configs
+MATRIX_HOMESERVER=
+MATRIX_ACCESSTOKEN=
+
 # syslog configs
 SYSLOG_FACILITY=
 
@@ -359,6 +370,19 @@ IRC_NICKNAME=
 IRC_REALNAME=
 IRC_NETWORK=
 
+# hangouts configs
+declare -A HANGOUTS_WEBHOOK_URI
+
+# dynatrace configs
+DYNATRACE_SPACE=
+DYNATRACE_SERVER=
+DYNATRACE_TOKEN=
+DYNATRACE_TAG_VALUE=
+DYNATRACE_ANNOTATION_TYPE=
+DYNATRACE_EVENT=
+SEND_DYNATRACE=
+
+
 # load the stock and user configuration files
 # these will overwrite the variables above
 
@@ -386,7 +410,7 @@ if [ -z ${EMAIL_CHARSET} ]; then
 fi
 
 # If we've been asked to use FQDN's for the URL's in the alarm, do so,
-# unless we're sending an alarm for a slave system which we can't get the
+# unless we're sending an alarm for a child system which we can't get the
 # FQDN of easily.
 if [ "${use_fqdn}" = "YES" ] && [ "${host}" = "$(hostname -s 2>/dev/null)" ]; then
 	host="$(hostname -f 2>/dev/null)"
@@ -428,7 +452,6 @@ filter_recipient_by_criticality() {
 		if [ -f "${NETDATA_CACHE_DIR}/alarm-notify/${method}/${r}/${alarm_id}" ]; then
 			# we do not remove the file, so that he will get future notifications of this alarm
 			debug "SEVERITY FILTERING for ${x} VIA ${method}: ALLOW: recipient has been notified for this alarm in the past (will still receive next status change)"
-			return 0
 		fi
 		;;
 
@@ -437,7 +460,6 @@ filter_recipient_by_criticality() {
 			# remove the file, so that he will only receive notifications for CRITICAL states for this alarm
 			rm "${NETDATA_CACHE_DIR}/alarm-notify/${method}/${r}/${alarm_id}"
 			debug "SEVERITY FILTERING for ${x} VIA ${method}: ALLOW: recipient has been notified for this alarm (will only receive CRITICAL notifications from now on)"
-			return 0
 		fi
 		;;
 	esac
@@ -491,10 +513,23 @@ filter_recipient_by_criticality() {
 # check irc
 [ -z "${IRC_NETWORK}" ] && SEND_IRC="NO"
 
+# check hangouts
+[ ${#HANGOUTS_WEBHOOK_URI[@]} -eq 0 ] && SEND_HANGOUTS="NO"
+
 # check fleep
 #shellcheck disable=SC2153
 { [ -z "${FLEEP_SERVER}" ] || [ -z "${FLEEP_SENDER}" ]; } && SEND_FLEEP="NO"
 
+# check dynatrace
+{ [ -z "${DYNATRACE_SPACE}" ] ||
+  [ -z "${DYNATRACE_SERVER}" ] ||
+  [ -z "${DYNATRACE_TOKEN}" ] ||
+  [ -z "${DYNATRACE_TAG_VALUE}" ] ||
+  [ -z "${DYNATRACE_EVENT}" ]; } && SEND_DYNATRACE="NO"
+
+# check matrix
+{ [ -z "${MATRIX_HOMESERVER}" ] || [ -z "${MATRIX_ACCESSTOKEN}" ]; } && SEND_MATRIX="NO"
+
 if [ "${SEND_PUSHOVER}" = "YES" ] ||
 	[ "${SEND_SLACK}" = "YES" ] ||
 	[ "${SEND_ROCKETCHAT}" = "YES" ] ||
@@ -511,8 +546,11 @@ if [ "${SEND_PUSHOVER}" = "YES" ] ||
 	[ "${SEND_KAFKA}" = "YES" ] ||
 	[ "${SEND_FLEEP}" = "YES" ] ||
 	[ "${SEND_PROWL}" = "YES" ] ||
+	[ "${SEND_HANGOUTS}" = "YES" ] ||
+	[ "${SEND_MATRIX}" = "YES" ] ||
 	[ "${SEND_CUSTOM}" = "YES" ] ||
-	[ "${SEND_MSTEAM}" = "YES" ]; then
+	[ "${SEND_MSTEAM}" = "YES" ] ||
+	[ "${SEND_DYNATRACE}" = "YES" ]; then
 	# if we need curl, check for the curl command
 	if [ -z "${curl}" ]; then
 		curl="$(command -v curl 2>/dev/null)"
@@ -536,7 +574,10 @@ if [ "${SEND_PUSHOVER}" = "YES" ] ||
 		SEND_KAFKA="NO"
 		SEND_FLEEP="NO"
 		SEND_PROWL="NO"
+		SEND_HANGOUTS="NO"
+		SEND_MATRIX="NO"
 		SEND_CUSTOM="NO"
+		SEND_DYNATRACE="NO"
 	fi
 fi
 
@@ -657,12 +698,16 @@ for method in "${SEND_EMAIL}" \
 	"${SEND_PD}" \
 	"${SEND_FLEEP}" \
 	"${SEND_PROWL}" \
+	"${SEND_MATRIX}" \
 	"${SEND_CUSTOM}" \
 	"${SEND_IRC}" \
+	"${SEND_HANGOUTS}" \
 	"${SEND_AWSSNS}" \
 	"${SEND_SYSLOG}" \
 	"${SEND_SMS}" \
-	"${SEND_MSTEAM}"; do
+	"${SEND_MSTEAM}" \
+    "${SEND_DYNATRACE}"; do
+	
 	if [ "${method}" == "YES" ]; then
 		proceed=1
 		break
@@ -1615,9 +1660,10 @@ send_irc() {
 		*) color="#777777" ;;
 		esac
 
+		SNDMESSAGE="${MESSAGE//$'\n'/", "}"
 		for CHANNEL in ${CHANNELS}; do
 			error=0
-			send_alarm=$(echo -e "USER ${NICKNAME} guest ${REALNAME} ${SERVERNAME}\\nNICK ${NICKNAME}\\nJOIN ${CHANNEL}\\nPRIVMSG ${CHANNEL} :${MESSAGE}\\nQUIT\\n" \  | nc "${NETWORK}" 6667)
+			send_alarm=$(echo -e "USER ${NICKNAME} guest ${REALNAME} ${SERVERNAME}\\nNICK ${NICKNAME}\\nJOIN ${CHANNEL}\\nPRIVMSG ${CHANNEL} :${SNDMESSAGE}\\nQUIT\\n" \  | nc "${NETWORK}" 6667)
 			reply_codes=$(echo "${send_alarm}" | cut -d ' ' -f 2 | grep -o '[0-9]*')
 			for code in ${reply_codes}; do
 				if [ "${code}" -ge 400 ] && [ "${code}" -le 599 ]; then
@@ -1667,6 +1713,51 @@ send_awssns() {
 	return 1
 }
 
+# -----------------------------------------------------------------------------
+# Matrix sender
+
+send_matrix() {
+	local homeserver="${1}" webhook accesstoken rooms="${2}" httpcode sent=0 payload
+
+	[ "${SEND_MATRIX}" != "YES" ] && return 1
+	[ -z "${MATRIX_ACCESSTOKEN}" ] && return 1
+
+	accesstoken="${MATRIX_ACCESSTOKEN}"
+
+	case "${status}" in
+	WARNING) emoji="⚠️" ;;
+	CRITICAL) emoji="🔴" ;;
+	CLEAR) emoji="✅" ;;
+	*) emoji="⚪️" ;;
+	esac
+
+	for room in ${rooms}; do
+		webhook="$homeserver/_matrix/client/r0/rooms/$(urlencode $room)/send/m.room.message?access_token=$accesstoken"
+		payload="$(
+	cat <<EOF
+	{
+		"msgtype": "m.notice",
+		"format": "org.matrix.custom.html",
+			"formatted_body": "${emoji} ${host} ${status_message} - <b>${name//_/ }</b><br>${chart} (${family})<br><a href=\"${goto_url}\">${alarm}</a><br><i>${info}</i>",
+			"body": "${emoji} ${host} ${status_message} - ${name//_/ } ${chart} (${family}) ${goto_url} ${alarm} ${info}"
+	}
+EOF
+		)"
+
+		httpcode=$(docurl -X POST --data "${payload}" "${webhook}")
+		if [ "${httpcode}" == "200" ]; then
+			info "sent Matrix notification for: ${host} ${chart}.${name} is ${status} to '${room}'"
+			sent=$((sent + 1))
+		else
+			error "failed to send Matrix notification for: ${host} ${chart}.${name} is ${status} to '${room}', with HTTP error code ${httpcode}."
+		fi
+	done
+
+	[ ${sent} -gt 0 ] && return 0
+
+	return 1
+}
+
 # -----------------------------------------------------------------------------
 # syslog sender
 
@@ -1770,6 +1861,159 @@ send_sms() {
 	return 1
 }
 
+# -----------------------------------------------------------------------------
+# hangouts sender
+
+send_hangouts() {
+	local rooms="${1}" httpcode sent=0 room color payload webhook
+
+	[ "${SEND_HANGOUTS}" != "YES" ] && return 1
+
+	case "${status}" in
+	WARNING) color="#ffa700" ;;
+	CRITICAL) color="#d62d20" ;;
+	CLEAR) color="#008744" ;;
+	*) color="#777777" ;;
+	esac
+
+	for room in ${rooms}; do
+		if [ -z "${HANGOUTS_WEBHOOK_URI[$room]}" ] ; then
+			info "Can't send Hangouts notification for: ${host} ${chart}.${name} to room ${room}. HANGOUTS_WEBHOOK_URI[$room] not defined"
+		else
+			webhook="${HANGOUTS_WEBHOOK_URI[$room]}"
+			payload="$(
+				cat <<EOF
+				{
+					"cards": [
+						{
+							"header": {
+								"title": "Netdata on ${host}",
+								"imageUrl": "${images_base_url}/images/banner-icon-144x144.png",
+								"imageStyle": "IMAGE"
+							},
+							"sections": [
+								{
+									"header": "<b>${host}</b>",
+									"widgets": [
+										{
+											"keyValue": {
+												"topLabel": "Status Message",
+												"content": "<b>${status_message}</b>",
+												"contentMultiline": "true",
+												"iconUrl": "${image}",
+												"onClick": {
+													"openLink": {
+														"url": "${goto_url}"
+													}
+												}
+											}
+										},
+										{
+											"keyValue": {
+												"topLabel": "${chart} | ${family}",
+												"content": "<font color=${color}>${alarm}</font>",
+												"contentMultiline": "true"
+											}
+										}
+									]
+								},
+								{
+									"widgets": [
+										{
+											"textParagraph": {
+											"text": "<font color=\"#0057e7\">@ ${date}\n<b>${info}</b></font>"
+											}
+										}
+									]
+								},
+								{
+									"widgets": [
+										{
+											"buttons": [
+												{
+													"textButton": {
+														"text": "Go to ${host}",
+														"onClick": {
+															"openLink": {
+																"url": "${goto_url}"
+															}
+														}
+													}
+												}
+											]
+										}
+									]
+								}
+							]
+						}
+					]
+				}
+EOF
+				)"
+
+			httpcode=$(docurl -H "Content-Type: application/json" -X POST -d "${payload}" "${webhook}")
+
+			if [ "${httpcode}" = "200" ]; then
+				info "sent hangouts notification for: ${host} ${chart}.${name} is ${status} to '${room}'"
+				sent=$((sent + 1))
+			else
+				error "failed to send hangouts notification for: ${host} ${chart}.${name} is ${status} to '${room}', with HTTP error code ${httpcode}."
+			fi
+		fi
+	done
+
+	[ ${sent} -gt 0 ] && return 0
+
+	return 1
+}
+# -----------------------------------------------------------------------------
+# Dynatrace sender
+send_dynatrace() {
+  [ "${SEND_DYNATRACE}" != "YES" ] && return 1
+
+  local dynatrace_url="${DYNATRACE_SERVER}/e/${DYNATRACE_SPACE}/api/v1/events"
+  local description="NetData Notification for: ${host} ${chart}.${name} is ${status}"
+  local payload=""
+  
+  payload=$(cat <<EOF
+{
+  "title": "NetData Alarm from ${host}",
+  "source" : "${DYNATRACE_ANNOTATION_TYPE}",
+  "description" : "${description}",
+  "eventType": "${DYNATRACE_EVENT}",
+   "attachRules":{
+     "tagRule":[{
+        "meTypes":["HOST"],
+        "tags":["${DYNATRACE_TAG_VALUE}"]
+     }]
+  },
+  "customProperties":{
+    "description": "${description}"
+  }
+}
+EOF
+)
+
+  # echo ${payload}
+
+  httpcode=$(docurl -X POST  -H "Authorization: Api-token ${DYNATRACE_TOKEN}"  -H "Content-Type: application/json" -d "${payload}"    ${dynatrace_url})
+  ret=$?
+
+
+  if [ ${ret} -eq 0 ]; then
+     if [ "${httpcode}" = "200" ]; then
+        info "sent ${DYNATRACE_EVENT} to ${DYNATRACE_SERVER}"
+        return 0
+     else
+        warning "Dynatrace ${DYNATRACE_SERVER} responded ${httpcode} notification for: ${host} ${chart}.${name} is ${status} was not sent!"
+        return 1
+     fi
+  else
+     error  "failed to sent ${DYNATRACE_EVENT} notification for: ${host} ${chart}.${name} is ${status} to  ${DYNATRACE_SERVER} with error code ${ret}."
+     return 1
+  fi
+}
+
 # -----------------------------------------------------------------------------
 # prepare the content of the notification
 
@@ -1888,6 +2132,15 @@ raised_for_html=
 send_slack "${SLACK_WEBHOOK_URL}" "${to_slack}"
 SENT_SLACK=$?
 
+# -----------------------------------------------------------------------------
+# send the hangouts notification
+
+# hangouts aggregates posts from the same room
+# so we use "${host} ${status}" as the room, to make them diff
+
+send_hangouts "${to_hangouts}"
+SENT_HANGOUTS=$?
+
 # -----------------------------------------------------------------------------
 # send the Microsoft notification
 
@@ -2084,6 +2337,13 @@ send_awssns "${to_awssns}"
 
 SENT_AWSSNS=$?
 
+# -----------------------------------------------------------------------------
+#  send the Matrix message
+send_matrix "${MATRIX_HOMESERVER}" "${to_matrix}"
+
+SENT_MATRIX=$?
+
+
 # -----------------------------------------------------------------------------
 # send the syslog message
 
@@ -2264,12 +2524,19 @@ fi
 
 SENT_EMAIL=$?
 
+# -----------------------------------------------------------------------------
+# send the EVENT to Dynatrace
+send_dynatrace "${host}" "${chart}" "${name}"  "${status}"
+SENT_DYNATRACE=$?
+
+
 # -----------------------------------------------------------------------------
 # let netdata know
 for state in "${SENT_EMAIL}" \
 	"${SENT_PUSHOVER}" \
 	"${SENT_TELEGRAM}" \
 	"${SENT_SLACK}" \
+	"${SENT_HANGOUTS}" \
 	"${SENT_ROCKETCHAT}" \
 	"${SENT_ALERTA}" \
 	"${SENT_FLOCK}" \
@@ -2286,9 +2553,11 @@ for state in "${SENT_EMAIL}" \
 	"${SENT_CUSTOM}" \
 	"${SENT_IRC}" \
 	"${SENT_AWSSNS}" \
+	"${SENT_MATRIX}" \
 	"${SENT_SYSLOG}" \
 	"${SENT_SMS}" \
-	"${SENT_MSTEAM}"; do
+	"${SENT_MSTEAM}" \
+    "${SENT_DYNATRACE}"; do	
 	if [ "${state}" -eq 0 ]; then
 		# we sent something
 		exit 0
